from typing import Any

import boto3
from cryptography.hazmat.primitives.serialization import load_pem_public_key
from jwt.algorithms import RSAAlgorithm


class KMSAlgorithm(RSAAlgorithm):
    region = "ap-northeast-1"

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        self.hash_alg = RSAAlgorithm.SHA256

    def get_client(self) -> boto3.session.Session.client:
        client = boto3.client("kms", self.region)
        return client

    def prepare_key(self, key: str) -> str:
        return key

    def sign(self, msg: str, key: str) -> str:
        client = self.get_client()
        resp = client.sign(
            KeyId=key, Message=msg, SigningAlgorithm="RSASSA_PKCS1_V1_5_SHA_256"
        )
        return resp["Signature"]  # type: ignore

    def verify(self, msg: str, key: bytes, sig: str) -> bool:
        real_key = load_pem_public_key(key)
        return super().verify(msg, real_key, sig)  # type: ignore



from typing import Any, Dict

import jwt

from revjwt.cert import prod_cert, stg_cert


class RevJWKClient(jwt.PyJWKClient):
    """
    monkey patch PyJWKClient, to use static keys rather than fetching
    """

    def __init__(self, data: Any) -> None:
        # explicitly pass "alg" and "use" field, so PyJWK can work normally
        _data = {
            "keys": [{"alg": "RS256", "use": "sig", **key} for key in data.get("keys")]
        }
        self._data = _data

    def fetch_data(self) -> Any:
        return self._data


def decode_static(token: str) -> Dict[str, Any]:
    """
    use static cert rather than fetched ones to perform jwt decode
    """

    decoded = jwt.decode(token, options={"verify_signature": False})
    jwk_client = RevJWKClient(stg_cert if decoded["env"] == "stg" else prod_cert)
    signing_key = jwk_client.get_signing_key_from_jwt(token)

    # TODO: verify the audience
    return jwt.decode(
        token, signing_key.key, algorithms=["RS256"], options={"verify_aud": False}
    )



import json
from typing import Any, Dict, List

import requests
from jwcrypto.jwk import JWK
from jwt.api_jws import PyJWS
from jwt.exceptions import DecodeError

from revjwt.algorithms import KMSAlgorithm

PUB_URL = "https://keys.revtel-api.com/certs.json"
PUB_STG_URL = "https://keys.revtel-api.com/certs-stg.json"

_keys: Dict[str, Dict[str, Any]] = {}


def fetch_key(kid: str, stage: str) -> Dict[str, Any]:
    if stage == "stg":
        url = PUB_STG_URL
    else:
        url = PUB_URL

    resp = requests.get(url).json()
    return [key for key in resp["keys"] if key["kid"] == kid][0]  # type: ignore


def get_key(kid: str, stage: str) -> Dict[str, Any]:
    try:
        return _keys[kid]
    except KeyError:
        key = fetch_key(kid, stage=stage)
        _keys[kid] = key
        return key


class JWS(PyJWS):
    def __init__(self, options: Any = None) -> None:
        super().__init__(options)  # type: ignore
        self._algorithms = {"RS256": KMSAlgorithm()}

    def decode_complete(
        self,
        jwt: str,
        key: str = "",
        algorithms: List[str] = ["RS256"],
        options: Any = None,
        **kwargs: Any,
    ) -> Dict[str, Any]:
        if options is None:
            options = {}
        merged_options = {**self.options, **options}
        verify_signature = merged_options["verify_signature"]

        if verify_signature and not algorithms:
            raise DecodeError(
                'It is required that you pass in a value for the "algorithms" argument when calling decode().'
            )

        payload, signing_input, header, signature = self._load(jwt)  # type: ignore

        json_payload = json.loads(payload.decode())
        kid = header["kid"]

        env = json_payload["env"]
        key = get_key(kid, stage=env)  # type: ignore

        key_json = JWK.from_json(json.dumps(key))
        key_pem = key_json.export_to_pem()

        self._verify_signature(signing_input, header, signature, key_pem, algorithms)  # type: ignore

        return {
            "payload": payload,
            "header": header,
            "signature": signature,
        }


_jws = JWS()
encode = _jws.encode
decode = _jws.decode_complete
